package logger

import (
	"database/sql"
	"fmt"
	"strings"
	"sync"
	"time"

	_ "github.com/mattn/go-sqlite3"
)

type LoggerInterface interface {
	Log(level LogLevel, message string)
	FetchLogs(limit int, levels []string) ([]LogEntry, error)
	Error(format string, args ...any)
	Info(format string,  args ...any)
	Warning(format string, args ...any)
	Debug(format string, args ...any)
	Fatal(format string, args ...any)
	Close() error
}

const (
    Debug   LogLevel = "debug"
    Info    LogLevel = "info"
    Warning LogLevel = "warning"
    Error   LogLevel = "error"
    Fatal   LogLevel = "fatal"
)

type LogLevel string

type LogEntry struct {
	Message 	string
	Timestamp 	string
	Level 		LogLevel
}

type Logger struct {
	db 		*sql.DB
	lock 	sync.Mutex
}

func New(db *sql.DB) *Logger {
	return &Logger{db: db}
}


func Init(dbPath string) (*Logger, error) {
	db, err := sql.Open("sqlite3", dbPath)
	if err != nil {
		return nil, fmt.Errorf("Error opening DB: %w", err)
	}

	CreateTableQuery := `CREATE TABLE IF NOT EXISTS logs (
		id INTEGER PRIMARY KEY AUTOINCREMENT,
		message TEXT,
		timestamp DATETIME DEFAULT CURRENT_TIMESTAMP,
		level TEXT
	);`

	if _, err = db.Exec(CreateTableQuery); err != nil {
		return nil, err
	}

	return New(db), nil
}

func (l *Logger) InsertLog(level LogLevel, message string) error {
	l.lock.Lock()
	defer l.lock.Unlock()

	tx, err := l.db.Begin()
	if err != nil {
		return fmt.Errorf("InsertLog: start tx: %w", err)
	}

	insertQuery := `INSERT INTO logs (message, level) VALUES(?, ?)`
	/* String escaping is obsolete when logfmt will not be used */
	// message = strings.ReplaceAll(message, `"`, `\"`)

	_, err = tx.Exec(insertQuery, message, level)
	if err != nil {
		tx.Rollback()
		return fmt.Errorf("InsertLog: exec: %w", err)
	}

	if err := tx.Commit(); err != nil {
		return fmt.Errorf("InsertLog: commit: %w", err)
	}

	return nil
}

func (l *Logger) FetchLogs(limit int, levels []string) ([]LogEntry, error) {
	l.lock.Lock()
	defer l.lock.Unlock()

	if len(levels) == 0 {
		levels = []string{"%"}
	}

	var args[] interface {}
	placeholders := make([]string, len(levels))
	for i, level := range levels{
		placeholders[i] = "level LIKE ?"
		args = append(args, level)
	}

	args = append(args, limit)

	query := fmt.Sprintf(`
		SELECT timestamp, level, message
		FROM logs
		WHERE %s
		ORDER BY timestamp DESC
		LIMIT ?`, strings.Join(placeholders, " OR "))

	rows, err := l.db.Query(query, args...)
	if err != nil {
		return nil, fmt.Errorf("FetchLogs: query: %w", err)
	}
	defer rows.Close()

	var logs []LogEntry
	for rows.Next() {
		var entry LogEntry
		if err := rows.Scan(&entry.Timestamp, &entry.Level, &entry.Message); err != nil {
			return nil, fmt.Errorf("FetchLogs: scan: %w", err)
		}
		logs = append(logs, entry)
	}

	return logs, nil
}

func (l * Logger) Close() error {
	return l.db.Close()
}

func ToLog(logLine string) string {
	log := fmt.Sprintf("%s",time.Now().Format(time.RFC3339) + " " + logLine)
	return log
}

func (l *Logger) Log(level LogLevel, message string) {
	_ = l.InsertLog(level, message)
}

func (l  *Logger) Error(format string, args ...any) {
	l.Log(Error, fmt.Sprintf(format, args...))
}

func (l  *Logger) Info(format string, args ...any) {
	l.Log(Info, fmt.Sprintf(format, args...))
}

func (l  *Logger) Warning(format string, args ...any) {
	l.Log(Warning, fmt.Sprintf(format, args...))
}

func (l *Logger) Debug(format string, args ...any) {
	l.Log(Debug, fmt.Sprintf(format, args...))
}

func (l *Logger) Fatal(format string, args ...any) {
	l.Log(Fatal, fmt.Sprintf(format, args...))
}
