// src/server/webapp/agents_handler_html_test.go
package webapp

import (
	"context"
	"net"
	"net/http"
	"net/http/httptest"
	"strings"
	"testing"
	"time"

	"github.com/DATA-DOG/go-sqlmock"
)

// startStubAgentNameServer spins up an HTTP server **on :5555**
// so getAgentsStatus() succeeds during tests.
func startStubAgentNameServer(t *testing.T) func() {
	t.Helper()

	// Prepare a listener on the exact address getAgentsStatus() calls.
	l, err := net.Listen("tcp", "127.0.0.1:5555")
	if err != nil {
		t.Fatalf("cannot bind :5555 (is another process using it?): %v", err)
	}

	mux := http.NewServeMux()
	mux.HandleFunc("/agentNames", func(w http.ResponseWriter, r *http.Request) {
		w.Header().Set("Content-Type", "application/json")
		w.Write([]byte(`["alpha","beta"]`))
	})

	srv := &http.Server{Handler: mux}
	go srv.Serve(l)

	// Return cleanup function.
	return func() {
		ctx, cancel := context.WithTimeout(context.Background(), time.Second)
		defer cancel()
		_ = srv.Shutdown(ctx)
	}
}

// Build App with sqlmock that returns an empty agents result.
func newAppForAgents(t *testing.T) *App {
	t.Helper()

	tmpl, err := ParseTemplates()
	if err != nil {
		t.Fatalf("ParseTemplates: %v", err)
	}

	// sqlmock with regexp matcher
	db, mock, err := sqlmock.New(sqlmock.QueryMatcherOption(sqlmock.QueryMatcherRegexp))
	if err != nil {
		t.Fatalf("sqlmock: %v", err)
	}

	// Any SELECT from agents returns zero rows
	mock.ExpectQuery("(?i)select .* from agents").
		WillReturnRows(sqlmock.NewRows([]string{"agent_id", "agent_name", "status"}))

	return &App{Tmpl: tmpl, DB: db}
}

func TestAgentsHandler_ListHTML(t *testing.T) {
	cleanup := startStubAgentNameServer(t)
	defer cleanup()

	app := newAppForAgents(t)

	req := httptest.NewRequest(http.MethodGet, "/agents", nil)
	rec := httptest.NewRecorder()

	app.agentsHandler(rec, req)

	if rec.Code != http.StatusOK {
		t.Fatalf("status = %d; want 200", rec.Code)
	}
	if ct := rec.Header().Get("Content-Type"); !strings.HasPrefix(ct, "text/html") {
		t.Errorf("Content‑Type = %q; want text/html", ct)
	}
	if !strings.Contains(rec.Body.String(), "<table") {
		t.Errorf("agents list HTML missing <table>")
	}
}
