package webapp

import (
    "net/http"
    "net/http/httptest"
    "os"
    "strings"
    "testing"

    "gontrol/src/logger"
)

func newTestAppWithSQLiteLogger(t *testing.T) *App {
    t.Helper()

    tmpl, err := ParseTemplates()
    if err != nil {
        t.Fatalf("ParseTemplates: %v", err)
    }

    tmp, err := os.CreateTemp("", "logs_*.db")
    if err != nil {
        t.Fatalf("Temp DB: %v", err)
    }
    tmp.Close()
    t.Cleanup(func() {
        os.Remove(tmp.Name())
    })

    logInstance, err := logger.Init(tmp.Name())
    if err != nil {
        t.Fatalf("logger.Init: %v", err)
    }
    t.Cleanup(func() {
        logInstance.Close()
    })

    return &App{
        Tmpl:   tmpl,
        Logger: logInstance,
    }
}

// Test that the logsHandler returns HTML with logs correctly
func TestLogsHandler_HTML(t *testing.T) {
    app := newTestAppWithSQLiteLogger(t)

    // Insert a log entry to be fetched
    app.Logger.Error("fake-log-1")

    req := httptest.NewRequest(http.MethodGet, "/logs/error?limit=1", nil)
    rec := httptest.NewRecorder()

    app.logsHandler(rec, req)

    if rec.Code != http.StatusOK {
        t.Fatalf("status = %d; want 200", rec.Code)
    }
    if ct := rec.Header().Get("Content-Type"); ct != "text/html; charset=utf-8" {
        t.Errorf("Content-Type = %s; want text/html", ct)
    }
    if !strings.Contains(rec.Body.String(), "fake-log-1") {
        t.Errorf("rendered HTML missing our fake log line")
    }
}

// Test that invalid 'limit' query param returns 400 Bad Request
func TestLogsHandler_InvalidLimit(t *testing.T) {
    app := newTestAppWithSQLiteLogger(t)

    req := httptest.NewRequest(http.MethodGet, "/logs?limit=abc", nil)
    rec := httptest.NewRecorder()

    app.logsHandler(rec, req)

    if rec.Code != http.StatusBadRequest {
        t.Fatalf("status = %d; want 400", rec.Code)
    }
    if !strings.Contains(rec.Body.String(), "Invalid count value") {
        t.Errorf("missing error message")
    }
}

// Test JSON output when client sends Accept: application/json
func TestLogsHandler_JSON(t *testing.T) {
    app := newTestAppWithSQLiteLogger(t)

    // Insert a log entry to be fetched
    app.Logger.Info("unit-test-json")

    req := httptest.NewRequest(http.MethodGet, "/logs?limit=1", nil)
    req.Header.Set("Accept", "application/json")
    rec := httptest.NewRecorder()

    app.logsHandler(rec, req)

    if rec.Code != http.StatusOK {
        t.Fatalf("status = %d; want 200", rec.Code)
    }
    if ct := rec.Header().Get("Content-Type"); ct != "application/json" {
        t.Errorf("Content-Type = %q; want application/json", ct)
    }
    if !strings.Contains(rec.Body.String(), "unit-test-json") {
        t.Errorf("JSON body missing expected log entry")
    }
}
