const checkboxState = new Map();

document.addEventListener('DOMContentLoaded', () => {

    document.body.addEventListener('htmx:beforeSwap', function(event) {
        if (event.target.id === "agentList") {
            saveCheckboxState();
        }
    });

    document.body.addEventListener('htmx:afterSwap', function(event) {
        if (event.target.id === "agentList") {
            restoreCheckboxState();
            updateAgentStatus();
            bindRowClicks();
        }

    // Stylize the results inside commandOutput
    const commandOutput = document.getElementById('commandOutput');
    styleCommandOutput();
    });

    bindRowClicks();
    restoreCheckboxState();

    themeToggle();
    focusCommandInput();
});

let cachedAgentNames = '';

function prepareAgentNames(event) {
    const form = event.target;

    // Only set agentNames if this is the navbar form
    const isNavbarForm = form.id === 'agentCommands';

    // If this is the first submission (navbar), calculate agentNames
    if (isNavbarForm) {
        const selected = Array.from(document.querySelectorAll('.agent-checkbox'))
            .filter(cb => cb.checked)
            .map(cb => cb.dataset.agentName);

        cachedAgentNames = selected.join(',');

        const hiddenInput = form.querySelector('[name="agentNames"]');
        const agentSelect = form.querySelector('select');

        if (selected.length > 0) {
            if (agentSelect) agentSelect.removeAttribute('name');
            hiddenInput.value = cachedAgentNames;
        } else {
            if (agentSelect) agentSelect.setAttribute('name', 'agentName');
            hiddenInput.value = '';
        }
    } else {
        // Use cached value for modal form
        const hiddenInput = form.querySelector('[name="agentNames"]');
        hiddenInput.value = cachedAgentNames;
    }
}

function toggleAllCheckboxes() {
    const checkboxes = document.querySelectorAll('input[name="agent-checkbox"]');
    const allChecked = Array.from(checkboxes).every(checkbox => checkbox.checked);

    checkboxes.forEach(checkbox => checkbox.checked = !allChecked);
    saveCheckboxState();

}

function saveCheckboxState() {
    document.querySelectorAll('.agent-checkbox').forEach((checkbox) => {
        checkboxState.set(checkbox.dataset.agentName, checkbox.checked);
    });
}

function restoreCheckboxState() {
    document.querySelectorAll('.agent-checkbox').forEach((checkbox) => {
        const state = checkboxState.get(checkbox.dataset.agentName);
        if (state !== undefined) {
            checkbox.checked = state;
        }
    });
}

// Because of this function, you can click anywhere on the row to select it
function bindRowClicks() {
    const rows = document.querySelectorAll('#agentList tbody tr');
    rows.forEach(row => {
        const agentName = row.dataset.agentName || row.cells[1]?.textContent?.trim();
        row.dataset.agentName = agentName;

        row.addEventListener('click', (e) => {
            if (e.target.tagName === "BUTTON" || e.target.type === "checkbox") return;

            const checkbox = row.querySelector(`input.agent-checkbox[data-agent-name="${agentName}"]`);
            if (checkbox) {
                checkbox.checked = !checkbox.checked;
                saveCheckboxState();
            }
        });
    });
}


// Colorize connection status inside agent list
function updateAgentStatus() {
    const select = document.getElementById("agentName");
    const optionValues = Array.from(select.options).map(opt => opt.value);
    const rows = document.querySelectorAll("#agentList tbody tr");

    rows.forEach(row => {
        const status = row.cells[4].textContent.trim();
        const name = row.cells[1].textContent.trim();

        if (status === "Connected") {
            row.cells[4].innerHTML = '<span class="badge bg-success">Connected</span>';
            const option = document.createElement("option");
            if (!(optionValues.includes(name))) {
                option.value = name;
                option.textContent = name;
                select.appendChild(option);
            }
        }

        if (status === "Disconnected") {
            row.cells[4].innerHTML = '<span class="badge bg-danger">Disconnected</span>';
            const option = Array.from(select.options).find(opt => opt.value === name);
            if(option) {
                select.removeChild(option);
            }
        }
    });
}

////////////////////////////////////////////////////////////////////////////////
//
// Toggle them icon from sun/moon on button click
//
////////////////////////////////////////////////////////////////////////////////

function themeToggle() {
  const body = document.body;
  const toggleBtn = document.getElementById('themeToggleButton');

  function applyTheme(theme) {
    body.setAttribute('data-bs-theme', theme);
    body.classList.remove('bg-light', 'text-dark', 'bg-dark', 'text-light');
    if (theme === 'dark') {
      body.classList.add('bg-dark', 'text-light');
      toggleBtn.innerHTML = '☽'; // Could switch to the colored version 🌙
    } else {
      body.classList.add('bg-light', 'text-dark');
      toggleBtn.innerHTML = '☼'; // Could switch to the colored version ☀
    }
  }

  const savedTheme = localStorage.getItem('theme') || 'light';
  applyTheme(savedTheme);

  toggleBtn.addEventListener('click', () => {
    const currentTheme = body.getAttribute('data-bs-theme');
    const newTheme = currentTheme === 'dark' ? 'light' : 'dark';
    applyTheme(newTheme);
  });
}

////////////////////////////////////////////////////////////////////////////////
//
// Colorize CommandOutput
//
////////////////////////////////////////////////////////////////////////////////


function styleCommandOutput() {
  // Split lines by newline character, since <pre> preserves formatting
  // let lines = document.getElementById('commandOutput').innerHTML.split('\n');
    let lines = commandOutput.innerHTML.split(/<br\s*\/?>|\n/);

  // Map lines: if contains "Error", wrap in red span, else plain text
  lines = lines.map(line => {
    if (line.includes("ERROR")) {
      return `<span style="color: var(--bs-danger);">${line}</span>`;
    } else if (/^\[\S*\]$/.test(line)) {
        return `<span style="color: var(--bs-primary); font-size: 16px">${line}</span>`;
    } else {
        return `<span style="font-size: 16px; line-height: 1em;">${line}</span>`;
    }
  });

  // Replace innerHTML with the joined lines, joined by <br> for line breaks in HTML
  commandOutput.innerHTML = lines.join('<br>');
}

////////////////////////////////////////////////////////////////////////////////
//
// Focus on modal command inpu
//
////////////////////////////////////////////////////////////////////////////////

function focusCommandInput() {
    // const modalEl = document.getElementById('exampleModal');
    var modalElement = document.getElementById('exampleModal')
    modalElement.addEventListener('shown.bs.modal', event => {
      document.getElementById('modalCommand').focus();
    })
}
